#### Visual modelling based on light conditions of second field experiment ####
## Only uses blue spark, split by north and south orientation ##

library(pavo)

rm(list = ls()) #clean workspace

#### Function to compute detection distance of scorpionfish pupil with and without ocular spark ####

Sp.OS <- function(pupil_rad_sp_north, pupil_rad_sp_south,
                  solid_angle_Sp, solid_angle_Os,
                  Iwd_N, Iwd_S, Isw_N, Isw_S, OScb, Rre, Ptd, Td, dist){
  
  # Radiance without Os
  
  # Attenuation of radiance based on attenuation coefficient and distance -> EQUATION 1 in visual model details
  Pupil_rad_north_attenuated <- pupil_rad_sp_north[,-1] * exp(-k[,2]*dist) 
  Pupil_rad_south_attenuated <- pupil_rad_sp_south[,-1] * exp(-k[,2]*dist) 
  
  # Radiance that hits the triplefin pupil -> EQUATION 2 in visual model details
  Pupil_rad_north_Td <- Pupil_rad_north_attenuated * solid_angle_Sp * (pi * Ptd^2)
  Pupil_rad_south_Td <- Pupil_rad_south_attenuated * solid_angle_Sp * (pi * Ptd^2)
  
  # Radiance with Os
  
  # Blue spark radiance 
  OSb_N <- Idw_N[,2] * OScb[,2]
  OSb_S <- Idw_S[,2] * OScb[,2]
  
  # Attenuated spark radiance that reaches the scorpionfish -> EQUATION 3 in visual model details
  OSb_dist_N <- OSb_N * exp(-k[,2]*dist) 
  OSb_dist_S <- OSb_S * exp(-k[,2]*dist) 
  
  # Proportion of spark radiance reflected from the scorpionfish eye -> EQUATION 4 in visual model details
  OSb_Sp_N <- OSb_dist_N * solid_angle_Os * Rre[,2] * pi^-1
  OSb_Sp_S <- OSb_dist_S * solid_angle_Os * Rre[,2] * pi^-1
  
  # Attentuate spark radiance travelling towards triplefin pupil -> EQUATION 5 in visual model details
  OSb_Sp_Td_N <- OSb_Sp_N * exp(-k[,2]*dist)
  OSb_Sp_Td_S <- OSb_Sp_S * exp(-k[,2]*dist)
  
  # Spark radiance that hits back the triplefin pupil -> EQUATION 5 in visual model details
  Pupil_rad_OSb_N <- OSb_Sp_Td_N * solid_angle_Sp * (pi * Ptd^2)
  Pupil_rad_OSb_S <- OSb_Sp_Td_S * solid_angle_Sp * (pi * Ptd^2)
  
  # Total radiance with ocular spark -> SUM of EQUATIONS 2 and 5
  Total_Rad_North_OSb <- Pupil_rad_north_Td + Pupil_rad_OSb_N
  Total_Rad_South_OSb <- Pupil_rad_south_Td + Pupil_rad_OSb_S
  
  # Computing quantum catches with and without ocular spark
  Radiances <- as.rspec(cbind(400:700,Pupil_rad_north_Td,Pupil_rad_south_Td,
                              Total_Rad_North_OSb, Total_Rad_South_OSb))
  
  Radiances_vismodel_N <- vismodel(rspecdata = Radiances,visual = Td, achromatic = 'ml',   # computing quantum catches for triplefins visual system, north orientation
                                 illum = 'ideal', vonkries = T, bkg = Isw_N[,2],relative = FALSE) 
  
  Radiances_vismodel_S <- vismodel(rspecdata = Radiances,visual = Td, achromatic = 'ml',   # computing quantum catches for triplefins visual system, south orientation
                                 illum = 'ideal', vonkries = T, bkg = Isw_S[,2],relative = FALSE) 
  
  Radiances_coldist_N <- coldist(modeldata = Radiances_vismodel_N, noise = 'neural',achro = F, # computing chromatic contrasts for north orientation (not shown in paper)
                               n = c(1,4,4), weber = 0.05, weber.ref = 3)
  
  Radiances_coldist_S <- coldist(modeldata = Radiances_vismodel_S, noise = 'neural',achro = F, # computing chromatic contrasts for south orientation (not shown in paper)
                                 n = c(1,4,4), weber = 0.05, weber.ref = 3)
  
  ## Calculating Michelson contrasts
  MC_North <- (max(Radiances_vismodel_N[c(1,3),4]) - min(Radiances_vismodel_N[c(1,3),4])) / sum(Radiances_vismodel_N[c(1,3),4]) # for north orientation
  MC_South <- (max(Radiances_vismodel_S[c(2,4),4]) - min(Radiances_vismodel_S[c(2,4),4])) / sum(Radiances_vismodel_S[c(2,4),4]) # for south orientation
  
  # Extract relevant chromatic contrasts only 
  chromatic_coldist_N <- rbind(Radiances_coldist_N[2,], Radiances_coldist_N[5,]) # for north orientation, not shown in paper
  chromatic_coldist_S <- rbind(Radiances_coldist_S[2,], Radiances_coldist_S[5,]) # for south orientation, not shown in paper
  
  # Save Michelson contrasts
  res <- list(MC_North,MC_South) 
  res
}


#### Load paramenters ####

#### Light fields ####

light_N <- read.csv('06a Light field 10m north.csv', header = T) # these are RADIANCEs
Idw_N <- as.rspec(light_N[,c(1,2,3,4)], lim = c(400,700)) # Isolate downwelling light field
Isw_N <- as.rspec(light_N[,c(1,5,6,7)], lim = c(400,700)) # Isolted sidewelling light field

light_S <- read.csv('06b Light field 10m south.csv', header = T) # these are RADIANCEs
Idw_S <- as.rspec(light_S[,c(1,2,3,4)], lim = c(400,700)) # Isolate downwelling light field
Isw_S <- as.rspec(light_S[,c(1,5,6,7)], lim = c(400,700)) # Isolted sidewelling light field

Idw_mean_N <- aggspec(Idw_N, by = 3) # takes the averages of the 3 measurements 
Isw_mean_N <- aggspec(Isw_N, by = 3)

Idw_mean_S <- aggspec(Idw_S, by = 3) # takes the averages of the 3 measurements 
Isw_mean_S <- aggspec(Isw_S, by = 3)

# Absorption coefficient calcualted from measurements at different depths
k <- read.csv('07 Attenuation coefficient STARESO.csv', header = T) 

#### Radius porcus and triplefin ####

Rtd <- read.csv('08 Pupil and iris radius triplefin.csv',header = T) # Pupil radius of 35 triplefin
Rtd$pupil_radius <- Rtd$pupil_radius/1000 # trasnform from mm to m
Ptd <- mean(Rtd[,2]) # Average: 0.00078 m

#### Load triplefin visual system ####

ocdat <- read.csv('09 Ocular media transmission.csv',header = T)
ocdat <- ocdat$Om

Td <- sensmodel(c(468, 517, 530), range = c(400,700),beta = T,integrate = T)

#### Scorpionfish pupil ####

Psp <- read.csv('10 Radius pupil scorpionfish.csv',header = T) # Pupil radius of 35 triplefin
Psp$radius <- Psp$radius/100 # trasnform from cm to m
Psp <- mean(Psp[,2]) # Average: 2.6 mm

Pupil_rad_north_individuals <- read.csv('11 Baseline radiance 3 porcus facing north.csv', header = T) # north
names(Pupil_rad_north_individuals) <- c('wl','Pupil1_N', 'Pupil2_N', 'Pupil3_N')
Pupil_rad_north_individuals <- as.rspec(Pupil_rad_north_individuals, lim = c(400,700))
#plot(Pupil_rad_north_individuals,col = c(1,2,4))

Pupil_rad_south_individuals <- read.csv('12 Baseline radiance 3 porcus facing south.csv', header = T) # south
names(Pupil_rad_south_individuals) <- c('wl','Pupil1_S', 'Pupil2_S', 'Pupil3_S')
Pupil_rad_south_individuals <- as.rspec(Pupil_rad_south_individuals, lim = c(400,700))
#plot(Pupil_rad_south_individuals,col = c(1,2,4))

Pupil_rad_north_mean <- aggspec(as.rspec(Pupil_rad_north_individuals), by = 3) # takes the averages of the 3 measurements 
Pupil_rad_south_mean <- aggspec(as.rspec(Pupil_rad_south_individuals), by = 3) # takes the averages of the 3 measurements 

Rre <- read.csv('13 Scorpionfish retroreflectance.csv', header = T)

#### Ocular spark ####
Osp_r <- read.csv('14 Ocular spark radius.csv',header = T) # Ocular spark radius
Osp_r$spark_radius <- Osp_r$spark_radius/1000 # trasnform from mm to m
Osp_r <- mean(Osp_r[,2]) # Average: 0.1 mm

Os_conversion <- read.csv('15 Ocular spark conversion curve.csv', header = T)

#### Computing of solid angle assuming triplefin pupil is a point ####

distance <- seq(from = 0.01, to = 0.15, by = 0.001)

displ <- 1.09/1000 # displacement of 1.09 mm 
distance_displ <- sqrt((displ)^2 + distance^2) # distance between center of displaced diameter to original diameter
Osp_r_displ <- Osp_r * distance / distance_displ # similarity among triangles. original radius : radius projected on tangent sphere = distance_displ : original distance 

solid_angle <- function(target_size,distance){
  s_angle <- (pi * target_size^2) / distance^2 
  s_angle
}

solid_angle_porcus <- solid_angle(target_size = Psp, distance = distance)
solid_angle_spark <- solid_angle(target_size = Osp_r_displ, distance = distance_displ) # as function of displaced distance and new perceived area

#### Run function - Pupil ####

North.mat <- matrix(NA, nrow = 41, ncol = 41)
South.mat <- matrix(NA, nrow = 41, ncol = 41)

system.time(for (l in 1:41) {
  for (j in 1:41) {
    
    Os_vec <- Os_conversion[, c(1,l + 1)]
    Rre_vec <- Rre[, c(1,j + 1)]
    
    North.res <- NULL
    South.res <- NULL
    
    for (i in 1:length(distance)) {
      
      temp.res <-  Sp.OS(pupil_rad_sp_north = Pupil_rad_north_mean, pupil_rad_sp_south = Pupil_rad_south_mean,
                         solid_angle_Sp = solid_angle_porcus[i], solid_angle_Os = solid_angle_spark[i],
                         Iwd_N = Idw_mean_N, Iwd_S = Idw_mean_S, Isw_N = Isw_mean_N, Isw_S = Isw_mean_S, OScb = Os_vec, Rre = Rre_vec, Ptd, Td, dist = distance[i])
      
      North.res[i] <- temp.res[[1]]
      South.res[i] <- temp.res[[2]]
    }
    
    dist.north <- cbind(distance, North.res)
    North.mat[j,l] <- dist.north[min(which(dist.north[,2] < 0.008)),1]
    
    dist.south <- cbind(distance, South.res)
    South.mat[j,l] <- dist.south[min(which(dist.south[,2] < 0.008)),1]
  }
})

#user       system  elapsed 
#7563.401  121.961  7898.509 

write.table(North.mat, file = "Results matrix PUPIL North.csv", sep = ",", col.names = F)
write.table(South.mat, file = "Results matrix PUPIL South.csv", sep = ",", col.names = F)

North.mat <- as.matrix(read.csv('Results matrix PUPIL North.csv', header = F)[,-1])
South.mat <- as.matrix(read.csv('Results matrix PUPIL South.csv', header = F)[,-1])

#### Plot Figure 5 ####

library(viridis)
library(fields)
library(pavo)

mean.Os.reflectance <- colMeans(Os_conversion[,-1])
mean.retroreflectance <- colMeans(Rre[,-1])

range(mean.Os.reflectance)
range(mean.retroreflectance)
range(North.mat)

par(mar = c(5,5,4,1) + 0.1)

image.plot(x = mean.Os.reflectance, y = mean.retroreflectance, z = North.mat*100, zlim = c(2,15), frame.plot = F, yaxt = "n", xaxt = "n",
           col = viridis(n = 64), axis.args = list(at = c(2,4,6,8,10,12,14), labels = c('2','4','6','8','10','12','14'), cex.axis = 2), xlab = 'Ocular spark reflectance (prop.)', 
           ylab = expression(~italic(S.~porcus)~pupil~retroreflectance~(prop.)), legend.shrink = 1, legend.cex = 2, cex.lab = 2, cex.axis = 2, main = 'Triplefins facing north', cex.main = 2,legend.width = 3)
axis(side = 2, at = c(10,15,20), cex.axis = 2, las = 1)
axis(side = 1, at = c(0.8,1.4,2), cex.axis = 2)
box(lty = 1, lwd = 1)
box(legend, lwd = 2)
contour(x = mean.Os.reflectance, y = mean.retroreflectance, z = North.mat*100,levels = 3:11,
        lty = c(3), add = T, lwd = c(2,2,2,3,2,2,2,2),col = c("black","black","black","red","black","black","black","black"), labcex = 2, method = 'edge', drawlabels = F)

range(South.mat)

par(mar = c(5,5,4,1) + 0.1)

image.plot(x = mean.Os.reflectance, y = mean.retroreflectance, z = South.mat*100, zlim = c(2,15), frame.plot = T, yaxt = "n", xaxt = "n",
           col = viridis(n = 64), axis.args = list(at = c(2,4,6,8,10,12,14), labels = c('2','4','6','8','10','12','14'), cex.axis = 2), xlab = 'Ocular spark reflectance (prop.)', 
           ylab = expression(~italic(S.~porcus)~pupil~retroreflectance~(prop.)), legend.shrink = 1, legend.cex = 2, cex.lab = 2, cex.axis = 2, main = 'Triplefins facing south', cex.main = 2,legend.width = 3)
axis(side = 2, at = c(10,15,20), cex.axis = 2, las = 1)
axis(side = 1, at = c(0.8,1.4,2), cex.axis = 2)
contour(x = mean.Os.reflectance, y = mean.retroreflectance, z = South.mat*100, levels = 4:14,
        lty = c(3), add = T, lwd = c(2,2,2,2,3,2,2,2,2,2),col = c("black","black","black","black","red","black","black","black","black","black"), labcex = 2, method = 'edge', drawlabels = F)


#### Function to compute detection distance of scorpionfish iris with and without ocular spark ####

Iris.OS <- function(iris_ref, solid_angle_Sp, solid_angle_Os,
                  Iwd_N, Iwd_S, Isw_N, Isw_S, OScb, Ptd, Td, dist){
  
  # Radiance without Os
  
  # Attenuation of radiance based on attenuation coefficient and distance -> EQUATION 1 in visual model details
  Iris_rad_attenuated_N <- iris_ref[,2] * Isw_N[,2] * exp(-k[,2]*dist) 
  Iris_rad_attenuated_S <- iris_ref[,2] * Isw_S[,2] * exp(-k[,2]*dist) 
  
  # Radiance that hits the triplefin pupil -> EQUATION 2 in visual model details
  Iris_rad_Td_N <- Iris_rad_attenuated_N * solid_angle_Sp * (pi * Ptd^2)
  Iris_rad_Td_S <- Iris_rad_attenuated_S * solid_angle_Sp * (pi * Ptd^2)
  
  # Radiance with Os
  
  # Blue spark radiance 
  OSb_N <- Idw_N[,2] * OScb[,2]
  OSb_S <- Idw_S[,2] * OScb[,2]
  
  # Attenuated spark radiance that reaches the scorpionfish -> EQUATION 3 in visual model details
  OSb_dist_N <- OSb_N * exp(-k[,2]*dist) 
  OSb_dist_S <- OSb_S * exp(-k[,2]*dist) 
  
  # Proportion of spark radiance reflected from the scorpionfish eye -> EQUATION 4 in visual model details 
  OSb_Sp_N <- OSb_dist_N * solid_angle_Os * iris_ref[,2] * pi^-1
  OSb_Sp_S <- OSb_dist_S * solid_angle_Os * iris_ref[,2] * pi^-1
  
  # Attentuate spark radiance travelling towards triplefin pupil -> EQUATION 5 in visual model details
  OSb_Sp_Td_N <- OSb_Sp_N * exp(-k[,2]*dist)
  OSb_Sp_Td_S <- OSb_Sp_S * exp(-k[,2]*dist)
  
  # Spark radiance that hits back the triplefin pupil -> EQUATION 5 in visual model details
  Iris_rad_OSb_N <- OSb_Sp_Td_N * solid_angle_Sp * (pi * Ptd^2)
  Iris_rad_OSb_S <- OSb_Sp_Td_S * solid_angle_Sp * (pi * Ptd^2)
  
  # Total radiance with ocular spark -> SUM of EQUATIONS 2 and 5
  Total_Rad_N <- Iris_rad_Td_N + Iris_rad_OSb_N
  Total_Rad_S <- Iris_rad_Td_S + Iris_rad_OSb_S
  
  # Computing quantum catches with and without ocular spark
  Radiances_N <- as.rspec(cbind(400:700,Iris_rad_Td_N,Total_Rad_N))
  Radiances_S <- as.rspec(cbind(400:700,Iris_rad_Td_S,Total_Rad_S))
  
  Radiances_vismodel_N <- vismodel(rspecdata = Radiances_N,visual = Td, achromatic = 'ml',  # computing quantum catches for triplefins visual system, north orientation
                                 illum = 'ideal', vonkries = T, bkg = Isw_N[,2],relative = FALSE) 
  
  Radiances_vismodel_S <- vismodel(rspecdata = Radiances_S,visual = Td, achromatic = 'ml',
                                   illum = 'ideal', vonkries = T, bkg = Isw_S[,2],relative = FALSE)  # computing quantum catches for triplefins visual system, south orientation
  
  Radiances_coldist_N <- coldist(modeldata = Radiances_vismodel_N, noise = 'neural',achro = F, # computing chromatic contrasts for north orientation (not shown in paper)
                               n = c(1,4,4), weber = 0.05, weber.ref = 3)
  
  Radiances_coldist_S <- coldist(modeldata = Radiances_vismodel_S, noise = 'neural',achro = F, # computing chromatic contrasts for north orientation (not shown in paper)
                                 n = c(1,4,4), weber = 0.05, weber.ref = 3)
  
  ## Calculating Michelson contrasts
  MC_Iris_N <- (max(Radiances_vismodel_N[c(1,2),4]) - min(Radiances_vismodel_N[c(1,2),4])) / sum(Radiances_vismodel_N[c(1,2),4]) # for north orientation
  MC_Iris_S <- (max(Radiances_vismodel_S[c(1,2),4]) - min(Radiances_vismodel_S[c(1,2),4])) / sum(Radiances_vismodel_S[c(1,2),4]) # for south orientation
  
  # Extract relevant chromatic contrasts only 
  chromatic_coldist_N <- rbind(Radiances_coldist_N[2,], Radiances_coldist_N[5,]) # for north orientation, not shown in paper
  chromatic_coldist_S <- rbind(Radiances_coldist_S[2,], Radiances_coldist_S[5,]) # for south orientation, not shown in paper
  
  # Save Michelson contrasts
  res <- list(MC_Iris_N,MC_Iris_S) 
  res
}


#### Iris parameters ####

iris_reflectance <- read.csv('16 Iris reflectance.csv', header = T)

#### Run function - Iris ####

Iris.res.mat_N <- matrix(NA, nrow = 41, ncol = 41)
Iris.res.mat_S <- matrix(NA, nrow = 41, ncol = 41)

system.time(for (l in 1:41) {
  for (j in 1:41) {
    
    Os_vec <- Os_conversion[, c(1,l + 1)]
    iris_vec <- iris_reflectance[, c(1,j + 1)]
    
    iris.res_N <- NULL
    iris.res_S <- NULL
    
    for (i in 1:length(distance)) {
      
      temp.res <- Iris.OS(iris_ref = iris_vec,
                          solid_angle_Sp = solid_angle_porcus[i], 
                          solid_angle_Os = solid_angle_spark[i],
                          Iwd_N = Idw_mean_N, Iwd_S = Idw_mean_S, Isw_N = Isw_mean_N, Isw_S = Isw_mean_S,
                          OScb = Os_vec, Ptd, Td, dist = distance[i])
      
      iris.res_N[i] <- temp.res[[1]]
      iris.res_S[i] <- temp.res[[2]]
    }
    
    dist.iris_N <- cbind(distance, iris.res_N)
    Iris.res.mat_N[j,l] <- dist.iris_N[min(which(dist.iris_N[,2] < 0.008)),1]
    
    dist.iris_S <- cbind(distance, iris.res_S)
    Iris.res.mat_S[j,l] <- dist.iris_S[min(which(dist.iris_S[,2] < 0.008)),1]
  }
})

write.table(Iris.res.mat_N, file = "Results IRIS matrix North.csv", sep = ",", col.names = F) # results show no contribution of Iris for detection
write.table(Iris.res.mat_S, file = "Results IRIS matrix South.csv", sep = ",", col.names = F) # results show no contribution of Iris for detection